from datetime import datetime
from datetime import timedelta
import os
import psutil
import threading

try:
    import ZPublisherEventsBackport  # noqa
except ImportError:
    pass


def initialize(context):
    from zodbstats import patchObjectReader
    patchObjectReader()

    try:
        import collective.stats.transchain  # noqa
    except ImportError:
        pass


process = psutil.Process(os.getpid())
zero = timedelta(0)


def init_stats():

    setattr(STATS, 'stats', {
        'time-start': datetime.now(),
        'time-after-traverse': zero,
        'time-before-commit': zero,
        'time-end': zero,
        'transchain': zero,
        'memory': process.memory_info(),
        'modified': 0,
        'zodb-loads': [],
        'zodb-cached': [],
        'zodb-uncached': [],
    })

STATS = threading.local()



""" plone.transformchain time """
from collective.stats import STATS
from collective.stats import init_stats
from datetime import datetime
from plone.transformchain import zpublisher

orig_applyTransform = zpublisher.applyTransform


def applyTransform(request, body=None):
    t1 = datetime.now()
    res = orig_applyTransform(request, body)

    try:
        if getattr(STATS, 'stats', None) is None:
            init_stats()
        STATS.stats['transchain'] = datetime.now() - t1
    except:
        pass

    return res


zpublisher.applyTransform = applyTransform



import csv
import sys


def main():
    logfile = sys.argv[1]
    try:
        outfile = open(sys.argv[2], 'wb')
    except:
        outfile = open('stats.csv', 'wb')

    writer = csv.writer(outfile)
    writer.writerow((
        'url', 'time', 't traverse', 't commit', 't transchain', 'setstate',
        'total', 'total cached', 'modified', 'rss before', 'rss after'
    ))

    for line in open(logfile, 'rb'):
        if 'collective.stats' not in line:
            continue

        try:
            info = line.split('collective.stats')[-1]
            i1, i2, i3, i4 = [i.strip() for i in info.split('|')[1:]]
        except:
            continue

        tital, after_traverse, before_commit, transchain, loads, \
            loads_total, loads_cached, modified = i1.split()
        rss1, rss2 = [i.strip() for i in i4.split('RSS:')[-1].split('-')]
        url = i2.strip()

        print '%s | %s' % (i1, i2)
        writer.writerow(
            (url, tital, after_traverse, before_commit, transchain, loads,
             loads_total, loads_cached, modified, rss1, rss2))

    outfile.close()


if __name__ == '__main__':
    main()



import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.0.2'


setup(
    name='collective.stats',
    version=version,
    description="Zope low level stats per request.",
    long_description=open('README.rst').read() +
    open(os.path.join('docs', 'HISTORY.txt')).read(),
    classifiers=[
        "Environment :: Console",
        "Framework :: Plone",
        "Framework :: Zope2",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Zope Public License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Logging",
    ],
    author='Enfold Systems',
    author_email='contact@enfoldsystems.com',
    maintainer='Alex Clark',
    maintainer_email='aclark@aclark.net',
    url='http://github.com/collective/collective.stats',
    license='ZPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'ZODB3',
        'Zope2',
        'psutil',
    ],
    extras_require=dict(
        oldzope=['ZPublisherEventsBackport']),
    entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone

      [console_scripts]
      collective-stats = collective.stats.export:main
      """,
)



from collective.stats import STATS
from collective.stats import init_stats
from collective.stats import process
from datetime import datetime
from datetime import timedelta
from zope import component
import ZPublisher.interfaces
import logging
import os

logger = logging.getLogger('collective.stats')


@component.adapter(ZPublisher.interfaces.IPubStart)
def pubStartHandler(ev):
    init_stats()


@component.adapter(ZPublisher.interfaces.IPubAfterTraversal)
def pubAfterTraverseHandler(ev):
    if getattr(STATS, 'stats', None) is None:
        init_stats()
    STATS.stats['time-after-traverse'] = datetime.now() - STATS.stats['time-start']  # noqa


@component.adapter(ZPublisher.interfaces.IPubBeforeCommit)
def pubBeforeCommitHandler(ev):
    if getattr(STATS, 'stats', None) is None:
        init_stats()
    STATS.stats['time-before-commit'] = datetime.now() - STATS.stats['time-start']  # noqa

    try:
        ob = ev.request['PARENTS'][-1]
        conn = ob._p_jar
        STATS.stats['modified'] = len(conn._registered_objects)
    except:
        pass


@component.adapter(ZPublisher.interfaces.IPubSuccess)
def pubSucessHandler(ev):
    environ = ev.request.environ
    if getattr(STATS, 'stats', None) is None:
        init_stats()
    stats = STATS.stats
    stats['time-end'] = datetime.now() - stats['time-start']

    total = 0
    total_cached = 0
    t_total = timedelta()
    t_cached = timedelta()
    t_uncached = timedelta()

    for td in stats['zodb-cached']:
        total += 1
        total_cached += 1
        t_total = t_total + td
        t_cached = t_cached + td

    for td in stats['zodb-uncached']:
        total += 1
        t_total = t_total + td
        t_uncached = t_uncached + td

    loads = timedelta()
    for td in stats['zodb-loads']:
        loads = loads + td

    def printTD(td):
        s = td.seconds + td.microseconds / 1000000.0
        return '%2.4f' % s

    rss1 = stats['memory'][0] / 1024
    rss2 = process.memory_info()[0] / 1024

    info = (
        printTD(stats['time-end']),
        printTD(stats['time-after-traverse']),
        printTD(stats['time-before-commit']),
        printTD(stats['transchain']),
        printTD(loads),
        total,
        total_cached,
        stats['modified'],
        environ['REQUEST_METHOD'],
        environ['PATH_INFO'],
        printTD(t_total),
        printTD(t_cached),
        printTD(t_uncached),
        rss1,
        rss2
    )

    if os.getenv("COLLECTIVE_STATS_DISABLE_LOG") != "1":
        logger.info(
            '| %s %s %s %s %s %0.4d %0.4d %0.4d '
            '| %s:%s | t: %s, t_c: %s, t_nc: %s '
            '| RSS: %s - %s' % info
        )

    ev.request.response.setHeader(
        'x-stats', '%s %s %s %s %s %0.4d %0.4d %0.4d' % info[:8]
    )

    del STATS.stats



from ZODB import broken
from ZODB.Connection import Connection
from ZODB.broken import Broken
from ZODB.serialize import ObjectReader
from collective.stats import STATS
from collective.stats import init_stats
from datetime import datetime
from persistent import PickleCache

############
# ZODB Stats

has_new_ghost = hasattr(PickleCache, 'new_ghost')


def patchObjectReader():
    Connection.setstate = setstate
    ObjectReader.load_persistent = load_persistent

# [zoid, class name, from cache, time]


def load_persistent(self, oid, klass):
    # Quick instance reference.  We know all we need to know
    # to create the instance w/o hitting the db, so go for it!
    try:
        if getattr(STATS, 'stats', None) is None:
            init_stats()
        stats_cached = STATS.stats['zodb-cached']
        stats_uncached = STATS.stats['zodb-uncached']
    except:
        stats_cached = []
        stats_uncached = []

    t1 = datetime.now()

    obj = self._cache.get(oid, None)
    if obj is not None:
        stats_cached.append(datetime.now() - t1)
        return obj

    if isinstance(klass, tuple):
        klass = self._get_class(*klass)

    if issubclass(klass, Broken):
        # We got a broken class. We might need to make it
        # PersistentBroken
        if not issubclass(klass, broken.PersistentBroken):
            klass = broken.persistentBroken(klass)

    try:
        obj = klass.__new__(klass)
    except TypeError:
        # Couldn't create the instance.  Maybe there's more
        # current data in the object's actual record!
        stats_uncached.append(datetime.now() - t1)

        return self._conn.get(oid)

    if has_new_ghost:
        self._cache.new_ghost(oid, obj)
    else:
        obj._p_oid = oid
        obj._p_jar = self._conn
        obj._p_changed = None
        self._cache[oid] = obj

    stats_uncached.append(datetime.now() - t1)
    return obj


origSetstate = Connection.setstate


def setstate(self, obj):
    t1 = datetime.now()
    res = origSetstate(self, obj)
    try:
        STATS.stats['zodb-loads'].append(datetime.now() - t1)
    except:
        pass
    return res
